#!/usr/bin/env python3
# -*- coding: utf-8 -*-

print()
print("*** WITHOUT PSPRINT ***")
print("An output statement which informs the user")
print("This statement requests the user to act")
print("A debugging output useless to the user")
print()

from psprint import print
print()
print("*** WITH PSPRINT ***")
print("An output statement which informs the user", mark=1)
print("This statement requests the user to act", mark=2)
print("A debugging output useless to the user", mark='bug')
print ()



#!/usr/bin/env python3
# -*- coding:utf-8; mode:python -*-
#
# Copyright 2020 Pradyumna Paranjape
# This file is part of psprint.
#
# psprint is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# psprint is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with psprint.  If not, see <https://www.gnu.org/licenses/>.
#
'''
Errors and warnings
'''


class PSPrintError(Exception):
    '''
    Base error from PSPrint
    '''


class PSPrintWarning(Warning):
    '''
    Base warning category from PSPrint
    '''


class BadMark(PSPrintError):
    '''
    A ``mark`` supplied in ``config`` cannot be parsed

    Args:
        mark: passed mark
        config: config file that defined the mark
    '''
    def __init__(self, mark: str, config: str) -> None:
        super().__init__(f'''
        Prefix-mark {mark} from {config} couldn't be parsed
        ''')


class BadPref(Exception):
    '''
    Prefix Style declared incorrectly

    Args:
        key: key
        value: (Bad) value supplied

    '''
    def __init__(self, key, value):
        super().__init__(f'''
        Bad value {value} for {key}
        ''')


class BadStyle(Exception):
    '''
    Style declared incorrectly

    Args:
        key: key
        value: (Bad) value supplied

    '''
    def __init__(self, key, value):
        super().__init__(f'''
        Bad value {repr(value)} for style {key}
        ''')


class BadPrefix(BadPref):
    '''
    Prefix string Error

    '''
    def __init__(self, prefix):
        super().__init__('pref', prefix)


class BadShortPrefix(BadPref):
    '''
    Prefix short string Error

    '''
    def __init__(self, pref_s):
        super().__init__('pref_s', pref_s)


class BadColor(BadStyle):
    '''
    Color Error

    '''
    def __init__(self, color):
        super().__init__('color', color)


class BadGloss(BadStyle):
    '''
    Gloss Error

    '''
    def __init__(self, gloss):
        super().__init__('gloss', gloss)


class BadBGCol(BadStyle):
    '''
    Background Color Error

    '''
    def __init__(self, bgcol):
        super().__init__('bgcol', bgcol)


class KeyWarning(PSPrintWarning):
    '''
    Warning that a key was wrongly passed and has been interpreted as default
    '''


class ValueWarning(PSPrintWarning):
    '''
    Warning that a value was wrongly passed and has been interpreted as default
    '''



#!/usr/bin/env python3
# -*- coding:utf-8; mode:python -*-
#
# Copyright 2020 Pradyumna Paranjape
# This file is part of psprint.
#
# psprint is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# psprint is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with psprint.  If not, see <https://www.gnu.org/licenses/>.
#
'''
ANSI colors' and styles' definitions
'''


import os
import types
import yaml


with open(os.path.join(os.path.dirname(__file__), "ansi.yml"), "r") as stream:
    ANSI = types.SimpleNamespace(**yaml.safe_load(stream))
    '''
    ANSI codes' namespace

    Earlier, these values were imported from
    `colorama. <https://pypi.org/project/colorama/>`__

    However, such import may not be needed.

    ANSI Colors:

      * 0: black, k
      * 1: red, r
      * 2: green, g
      * 3: yellow, y
      * 4: blue, b
      * 5: magenta, m
      * 6: cyan, c
      * 7: white, w
      * 8 + `code`: light `color name`, l`c`
      * 16: terminal, t  (Terminal-determined)

    ANSI Gloss:

      * 0: reset all
      * 1: normal
      * 2: dim
      * 3: bright

    '''



#!/usr/bin/env python3
# -*- coding:utf-8; mode:python -*-
#
# Copyright 2020 Pradyumna Paranjape
# This file is part of psprint.
#
# psprint is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# psprint is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with psprint.  If not, see <https://www.gnu.org/licenses/>.
#
'''
Prompt String-like Print
'''

import os
import sys
import pathlib
from .printer import PrintSpace


# Initiate default print function
def init_print(custom: str = None) -> PrintSpace:
    '''
    Initiate ps-print function with default marks
    and marks read from various psprintrc configurarion files:

    Args:
        custom: custom configuration file location

    '''
    # psprintrc file locations
    user_home = pathlib.Path(os.environ["HOME"])
    config = user_home.joinpath(".config")  # default
    rc_locations = {
        'root': pathlib.Path("/etc/psprint/style.yml"),
        'user': user_home.joinpath("." + "psprintrc"),  # bad habit
        'config': pathlib.Path(
            os.environ.get("XDG_CONFIG_HOME", str(config))
        ).joinpath("psprint", "style.yml"),  # good habit
        'local': pathlib.Path('.').resolve().joinpath("." + "psprintrc"),
        'custom': None,
    }

    if custom is not None:
        rc_locations['custom'] = pathlib.Path(custom)

    default_config = os.path.join(os.path.dirname(__file__), "style.yml")
    default_print = PrintSpace(config=default_config)

    for loc in ('root', 'user', 'config', 'local', 'custom'):
        # DONT: loc from tuple, not keys(), deliberately to ascertain order
        if rc_locations[loc] is not None:
            if rc_locations[loc].exists():  # type: ignore
                default_print.set_opts(rc_locations[loc])

    if 'idlelib.run' in sys.modules or not sys.stdout.isatty():
        # Running inside idle
        default_print.switches['bland'] = True
    return default_print


DEFAULT_PRINT = init_print()
'''
PrintSpace object created by reading defaults from various
psprintrc and psprint/style.yml files

'''


print = DEFAULT_PRINT.psprint
'''
psprint function for imports

'''


__all__ = ['DEFAULT_PRINT', 'print']


__version__ = "21.3.1"



#!/usr/bin/env python3
# -*- coding:utf-8; mode:python -*-
#
# Copyright 2020 Pradyumna Paranjape
# This file is part of psprint.
#
# psprint is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# psprint is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with psprint.  If not, see <https://www.gnu.org/licenses/>.
#
'''
module init
'''


from . import print


if __name__ in ("__main__", "psprint.__main__"):
    print()
    print("usage:", mark='err', pad=True, short=False)
    print("Use me as an imported module", mark='info', pad=True, short=False)
    print("from psprint import PRINT as print",
          mark='act', pad=True, short=False)
    print("Styles may be edited by importing the PrintSpace class",
          mark='list', pad=True, short=False)
    print("Or by editing its DEFAULT_PRINT instance",
          pad=True, short=False)
    print("Bye", mark='bug', pad=True, short=False)
    print()



#!/usr/bin/env python3
# -*- coding:utf-8; mode:python -*-
#
# Copyright 2020 Pradyumna Paranjape
# This file is part of psprint.
#
# psprint is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# psprint is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with psprint.  If not, see <https://www.gnu.org/licenses/>.
#
'''
Information- Prepended Print object
'''

import sys
import typing
import yaml
from .ansi import ANSI
from .mark_types import InfoMark
from .errors import BadMark


class PrintSpace():
    '''
    Fancy Print class that also prints the type of message

    Args:
        config: path to default configuration file (shipped)

    Attributes:
        pref_max: int: maximum length of prefix string
        info_style; dict: pre-defined prefix styles
        info_index: list: keys of `info_style` mapped to int
        print_kwargs: dict : library of kwargs_accepted by print_function
        switches: dict: user-customizations: pad, short, bland, disabled

            pad: bool: prefix is padded to start text at the same level
            short: bool: display short, 1 character- prefix
            bland: bool: do not show ANSI color/styles for prefix/text
            disabled: bool: behave like python default print_function

    '''
    def __init__(self, config) -> None:
        # Standard info styles
        self.switches = {'pad': False, 'short': False,
                         'bland': False, 'disabled': False}
        self.print_kwargs = {'file': sys.stdout, 'sep': "\t",
                             'end': "\n", 'flush': False}
        self.pref_max = None
        self.info_style: typing.Dict[str, InfoMark] = {}
        self.info_index: typing.List[str] = []
        self.set_opts(config=config)

    def set_opts(self, config) -> None:
        '''
        Configure from rcfile

        Args:
            rcfile: .psprintrc file to read

        Raises:
            BadMark

        '''
        info_index: typing.Optional[typing.Dict[str, str]] = None
        with open(config, 'r') as rcfile:
            conf: typing.Dict[str, dict] = yaml.safe_load(rcfile)
        for mark, settings in conf.items():
            if mark == "FLAGS":
                # switches / flags
                self.pref_max = settings.get("pref_max_len", None)
                for b_sw in self.switches:
                    self.switches[b_sw] = settings.get(b_sw, False)
                self.print_kwargs['sep'] = settings.get("sep", "\t")
                self.print_kwargs['end'] = settings.get("end", "\n")
                self.print_kwargs['flush'] = settings.get("flush", False)
                fname = settings.get("file", None)  # Discouraged
                if fname is not None:
                    self.print_kwargs['file'] = open(fname, "a")
            elif mark == 'order':
                info_index = settings
            else:
                # Mark definition
                try:
                    self.edit_style(mark=mark, **settings)
                except ValueError:
                    raise BadMark(str(mark), rcfile.name) from None
        if info_index is not None:
            self.info_index = list(filter(lambda x: x in self.info_style,
                                          info_index))

    def edit_style(self, index_int: int = None,
                   mark: str = None, **kwargs) -> str:
        '''
        Edit loaded style

        Args:
            index_int: Index number that will call this InfoMark
            mark: Mark string that will call this ``InfoMark``
            **kwargs:
                * pref: str: prefix string long [length < 10 characters]
                * pref_s: str: prefix string short [1 character]
                * code:

                    * color: {[0-15],[[l]krgybmcw],[[light] color_name]}
                    * gloss: {[0-3],[rcdb],{reset,normal,dim,bright}}

                * for-

                    * pref_color: color of of prefix
                    * pref_gloss: gloss of prefix
                    * pref_bgcol: background color of prefix
                    * text_color: color of of text
                    * text_gloss: gloss of text
                    * text_bgcol: background color of text

        Returns
            Summary of new (updated) ``PrintSpace``

        '''
        # correct pref
        if kwargs['pref'] is None:
            raise ValueError(str)
        if mark is None:
            mark = kwargs['pref'][:4]
        if index_int is None or \
           not 0 <= index_int <= len(self.info_index):
            self.info_index.append(mark)
        else:
            self.info_index.insert(index_int, mark)
        self.info_style[mark] = InfoMark(pref_max=self.pref_max, **kwargs)
        return str(self)

    def remove_style(self, mark: str = None, index_int: int = None) -> str:
        '''
        Args:
        mark: is popped out of defined styles
        index_int: is used to locate index_str if it is not provided

        Returns
            Summary of new (updated) ``PrintSpace``

        '''
        if mark is None:
            if index_int is not None:
                if index_int < len(self.info_style):
                    mark = self.info_index.pop(index_int)
        if mark is None:
            raise SyntaxError('''
            At least one of mark and index_int should be provided
            ''')
        del self.info_style[mark]
        return str(self)

    def __str__(self) -> str:
        '''
        Returns:
            Formatted summary of info_style
        '''
        outstr = '\npref\tlong\tshort\ttext\n\n'
        outstr += "\n".join((f"{k}:{v}" for k, v in self.info_style.items()))
        return outstr

    def _which_mark(self, mark: typing.Union[str, int, InfoMark] = None,
                    **kwargs) -> InfoMark:
        '''
        Define a mark based on arguments supplied

        may be a pre-defined mark
        OR
        mark defined on the fly

        Args:
            mark: mark that identifies a defined prefix
            **kwargs:
                * pref: str: prefix string long [length < 10 characters]
                * pref_s: str: prefix string short [1 character]
                * code:

                    * color: {[0-15],[[l]krgybmcw],[[light] color_name]}
                    * gloss: {[0-3],[rcdb],{reset,normal,dim,bright}}

                * for-

                    * pref_color: color of of prefix
                    * pref_gloss: gloss of prefix
                    * pref_bgcol: background color of prefix
                    * text_color: color of of text
                    * text_gloss: gloss of text
                    * text_bgcol: background color of text

        '''
        base_mark: InfoMark = self.info_style['cont']
        if mark is not None:
            # Defined mark
            if isinstance(mark, InfoMark):
                base_mark = mark
            elif isinstance(mark, int):
                if not 0 <= mark < len(self.info_index):
                    mark = 0
                base_mark = self.info_style[self.info_index[mark]]
            elif isinstance(mark, str):
                base_mark = self.info_style.get(mark) or base_mark
            else:
                raise BadMark(mark=str(mark), config="**kwargs")
        if any(arg in kwargs for arg in [
                'pref', 'pref_s', 'pref_color',
                'pref_gloss', 'pref_bgcol', 'text_color', 'text_gloss',
                'text_bgcol',
        ]):
            return InfoMark(parent=base_mark, pref_max=self.pref_max, **kwargs)
        return base_mark

    def psprint(self, *args, mark: typing.Union[str, int, InfoMark] = None,
                **kwargs) -> None:
        '''
        Prefix String PRINT

        Args:
            *args: passed to print_function for printing
            mark: pre-declared InfoMark defaults:

                * cont or 0 or anything else: nothing
                * info or 1: [INFO]
                * act  or 2: [ACTION]
                * list or 3: [LIST]
                * warn or 4: [WARNING]
                * error:or 5: [ERROR]
                * bug:  or 6 [DEBUG]
                * `Other marks defined in .psprintrc`

            **kwargs:
                * pref: str: prefix string long [length < 10 characters]
                * pref_s: str: prefix string short [1 character]
                * code:

                    * color: {[0-15],[[l]krgybmcw],[[light] color_name]}
                    * gloss: {[0-3],[rcdb],{reset,normal,dim,bright}}

                * for-

                    * pref_color: color of of prefix
                    * pref_gloss: gloss of prefix
                    * pref_bgcol: background color of prefix
                    * text_color: color of of text
                    * text_gloss: gloss of text
                    * text_bgcol: background color of text

                * pad: bool: prefix is padded to start text at the same level
                * short: bool: display short, 1 character- prefix
                * bland: bool: do not show ANSI color/styles for prefix/text
                * disabled: bool: behave like python default print_function
                * file: typing.IO: passed to print function
                * sep: str: passed to print function
                * end: str: passed to print function
                * flush: bool: passed to print function

        Raises:
            BadMark
        '''
        # Extract kwargs
        print_kwargs = {key: {**self.print_kwargs, **kwargs}[key]
                        for key in self.print_kwargs}

        switches = {key: {**self.switches, **kwargs}[key]
                    for key in self.switches}
        if switches['disabled'] or not args:
            print(*args, **print_kwargs)  # type: ignore
            return

        mark = self._which_mark(mark=mark, **kwargs)

        # add prefix to *args[0]
        args_l = list(args)  # typecast
        if not switches.get('bland'):
            args_l[0] = str(mark.text) + str(args_l[0])
            args_l[-1] = str(args_l[-1]) + ANSI.RESET_ALL
        args_l[0] = mark.pref.to_str(**switches) + args_l[0]
        print(*args_l, **print_kwargs)  # type: ignore
